package kr.co.vividnext.sodalive.live.room.detail

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.common.Utils
import kr.co.vividnext.sodalive.live.LiveRepository

class LiveRoomDetailViewModel(private val repository: LiveRepository) : BaseViewModel() {
    private val _liveRoomDetailLiveData = MutableLiveData<GetRoomDetailResponse>()
    val liveRoomDetailLiveData: MutableLiveData<GetRoomDetailResponse>
        get() = _liveRoomDetailLiveData

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    fun getDetail(roomId: Long, onFailure: () -> Unit) {
        if (!_isLoading.value!!) {
            _isLoading.value = true
        }

        compositeDisposable.add(
            repository.getRoomDetail(roomId, token = "Bearer ${SharedPreferenceManager.token}")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success && it.data != null) {
                            _liveRoomDetailLiveData.postValue(it.data!!)
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                            onFailure()
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        onFailure()
                    }
                )
        )
    }

    fun shareRoomLink(
        roomId: Long,
        isPrivateRoom: Boolean,
        password: Int?,
        onSuccess: (String) -> Unit
    ) {
        val params = mapOf(
            "af_dp" to "voiceon://",
            "deep_link_value" to "live",
            "deep_link_sub5" to "$roomId"
        )

        val shareUrl = Utils.createOneLinkUrl(params = params)

        val message = if (isPrivateRoom) {
            "${SharedPreferenceManager.nickname}님이 귀하를 " +
                "보이스온의 비공개라이브에 초대하였습니다.\n" +
                "※ 라이브 참여: $shareUrl\n" +
                "(입장 비밀번호 : $password)"
        } else {
            "${SharedPreferenceManager.nickname}님이 귀하를 " +
                "보이스온의 공개라이브에 초대하였습니다.\n" +
                "※ 라이브 참여: $shareUrl"
        }

        onSuccess(message)
    }
}
